Class: Rouge::Lexers::Cpp
- Inherits:
-
C
- Object
- Rouge::Lexer
- RegexLexer
- C
- Rouge::Lexers::Cpp
- Defined in:
- lib/rouge/lexers/cpp.rb
Direct Known Subclasses
Constant Summary
Constants inherited from RegexLexer
Constants included from Token::Tokens
Token::Tokens::Num, Token::Tokens::Str
Instance Attribute Summary
Attributes inherited from Rouge::Lexer
Class Method Summary collapse
Methods inherited from C
Methods inherited from RegexLexer
append, #delegate, #fallthrough!, get_state, #get_state, #goto, #group, #groups, #in_state?, #pop!, prepend, #push, #recurse, replace_state, #reset!, #reset_stack, #stack, start, start_procs, state, #state, #state?, state_definitions, states, #step, #stream_tokens, #token
Methods inherited from Rouge::Lexer
aliases, all, #as_bool, #as_lexer, #as_list, #as_string, #as_token, assert_utf8!, #bool_option, continue_lex, #continue_lex, debug_enabled?, demo, demo_file, desc, detect?, detectable?, disable_debug!, eager_load!, #eager_load!, enable_debug!, filenames, find, find_fancy, guess, guess_by_filename, guess_by_mimetype, guess_by_source, guesses, #hash_option, #initialize, lazy, lex, #lex, #lexer_option, #list_option, lookup_fancy, mimetypes, option, option_docs, #reset!, skip_auto_load?, #stream_tokens, #string_option, tag, #tag, title, #token_option, #with
Methods included from Token::Tokens
Constructor Details
This class inherits a constructor from Rouge::Lexer
Class Method Details
.keywords ⇒ Object
23 24 25 26 27 28 29 30 31 32 33 |
# File 'lib/rouge/lexers/cpp.rb', line 23 def self.keywords @keywords ||= super + Set.new(%w( and and_eq asm bitand bitor catch compl concept consteval constinit const_cast co_await co_return co_yield decltype delete dynamic_cast explicit export final friend import module mutable namespace new noexcept not not_eq operator or or_eq override private protected public reinterpret_cast requires size_of static_cast this throw throws try typeid typename using virtual xor xor_eq )) end |
.keywords_type ⇒ Object
35 36 37 38 39 |
# File 'lib/rouge/lexers/cpp.rb', line 35 def self.keywords_type @keywords_type ||= super + Set.new(%w( char8_t )) end |
.reserved ⇒ Object
41 42 43 44 45 46 |
# File 'lib/rouge/lexers/cpp.rb', line 41 def self.reserved @reserved ||= super + Set.new(%w( __virtual_inheritance __uuidof __super __single_inheritance __multiple_inheritance __interface __event )) end |