Class: Rouge::Lexers::Hack
- Inherits:
-
PHP
- Object
- Rouge::Lexer
- RegexLexer
- TemplateLexer
- PHP
- Rouge::Lexers::Hack
- Defined in:
- lib/rouge/lexers/hack.rb
Constant Summary
Constants inherited from PHP
Constants inherited from RegexLexer
Constants included from Token::Tokens
Token::Tokens::Num, Token::Tokens::Str
Instance Attribute Summary
Attributes inherited from Rouge::Lexer
Class Method Summary collapse
Methods inherited from PHP
Methods inherited from TemplateLexer
Methods inherited from RegexLexer
append, #delegate, #fallthrough!, get_state, #get_state, #goto, #group, #groups, #in_state?, #pop!, prepend, #push, #recurse, replace_state, #reset!, #reset_stack, #stack, start, start_procs, state, #state, #state?, state_definitions, states, #step, #stream_tokens, #token
Methods inherited from Rouge::Lexer
aliases, all, #as_bool, #as_lexer, #as_list, #as_string, #as_token, assert_utf8!, #bool_option, continue_lex, #continue_lex, debug_enabled?, demo, demo_file, desc, detectable?, disable_debug!, eager_load!, #eager_load!, enable_debug!, filenames, find, find_fancy, guess, guess_by_filename, guess_by_mimetype, guess_by_source, guesses, #hash_option, #initialize, lazy, lex, #lex, #lexer_option, #list_option, lookup_fancy, mimetypes, option, option_docs, #reset!, skip_auto_load?, #stream_tokens, #string_option, tag, #tag, title, #token_option, #with
Methods included from Token::Tokens
Constructor Details
This class inherits a constructor from Rouge::Lexers::PHP
Class Method Details
.detect?(text) ⇒ Boolean
15 16 17 18 19 20 |
# File 'lib/rouge/lexers/hack.rb', line 15 def self.detect?(text) return true if text.start_with?('<?hh') return true if text.shebang?('hhvm') return false end |
.keywords ⇒ Object
22 23 24 25 26 27 28 29 30 31 |
# File 'lib/rouge/lexers/hack.rb', line 22 def self.keywords @keywords ||= super.merge Set.new %w( type newtype enum as super async await Awaitable vec dict keyset void int string bool float double arraykey num Stringish ) end |