Class: Rouge::Lexers::Lutin

Inherits:
Lustre show all
Defined in:
lib/rouge/lexers/lutin.rb

Constant Summary

Constants inherited from RegexLexer

RegexLexer::MAX_NULL_SCANS

Constants included from Token::Tokens

Token::Tokens::Num, Token::Tokens::Str

Instance Attribute Summary

Attributes inherited from Rouge::Lexer

#options

Class Method Summary collapse

Methods inherited from RegexLexer

append, #delegate, #fallthrough!, get_state, #get_state, #goto, #group, #groups, #in_state?, #pop!, prepend, #push, #recurse, replace_state, #reset!, #reset_stack, #stack, start, start_procs, state, #state, #state?, state_definitions, states, #step, #stream_tokens, #token

Methods inherited from Rouge::Lexer

aliases, all, #as_bool, #as_lexer, #as_list, #as_string, #as_token, assert_utf8!, #bool_option, continue_lex, #continue_lex, debug_enabled?, demo, demo_file, desc, detect?, detectable?, disable_debug!, eager_load!, #eager_load!, enable_debug!, filenames, find, find_fancy, guess, guess_by_filename, guess_by_mimetype, guess_by_source, guesses, #hash_option, #initialize, lazy, lex, #lex, #lexer_option, #list_option, lookup_fancy, mimetypes, option, option_docs, #reset!, skip_auto_load?, #stream_tokens, #string_option, tag, #tag, title, #token_option, #with

Methods included from Token::Tokens

token

Constructor Details

This class inherits a constructor from Rouge::Lexer

Class Method Details

.keywordsObject



17
18
19
20
21
22
# File 'lib/rouge/lexers/lutin.rb', line 17

def self.keywords
  @keywords ||= Set.new %w(
    let in node extern system returns weak strong assert raise try catch
    trap do exist erun run type ref exception include false true
  )
end

.primitivesObject



29
30
31
# File 'lib/rouge/lexers/lutin.rb', line 29

def self.primitives
  @primitives ||= Set.new %w(int real bool trace loop fby)
end

.word_operatorsObject



24
25
26
27
# File 'lib/rouge/lexers/lutin.rb', line 24

def self.word_operators
  @word_operators ||= Set.new %w(
     div and xor mod or not nor if then else pre)
end