Exception: Rouge::RegexLexer::Fallthrough

Inherits:
Exception
  • Object
show all
Defined in:
lib/rouge/regex_lexer.rb

Overview

exception class for fallthrough - using an exception is slightly faster than catch { … }, but it is not semantically an error that should be rescued from.