Exception: Rouge::RegexLexer::Fallthrough
- Inherits:
-
Exception
- Object
- Exception
- Rouge::RegexLexer::Fallthrough
- Defined in:
- lib/rouge/regex_lexer.rb
Overview
exception class for fallthrough - using an exception is slightly faster than catch { … }, but it is not semantically an error that should be rescued from.